#!/bin/sh

# Copyright (c) 2015-2023 the original authors.
#
# Based on https://github.com/gradle/gradle/blob/master/subprojects/plugins/src/main/resources/org/gradle/api/internal/plugins/unixStartScript.txt.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#   Important for running:
#
#       You need a POSIX-compliant shell to run this script. If your /bin/sh is
#       noncompliant, but you have some other compliant shell such as ksh or
#       bash, then to run this script, type that shell name before the whole
#       command line, like:
#
#           ksh saymon-agent.sh
#
#       Busybox and similar reduced shells will NOT work, because this script
#       requires all of these POSIX shell features:
#         * functions;
#         * expansions «$var», «${var}», «${var:-default}», «${var+SET}»,
#           «${var#prefix}», «${var%suffix}», and «$( cmd )»;
#         * compound commands having a testable exit status, especially «case»;
#         * various built-in commands including «command», «set», and «ulimit».

# Attempt to set APP_HOME

# Resolve links: $0 may be a link
app_path=$0

# Need this for daisy-chained symlinks.
while
    APP_HOME=${app_path%"${app_path##*/}"}  # leaves a trailing /; empty if no leading path
    [ -h "$app_path" ]
do
    ls=$( ls -ld "$app_path" )
    link=${ls#*' -> '}
    case $link in             #(
      /*)   app_path=$link ;; #(
      *)    app_path=$APP_HOME$link ;;
    esac
done

APP_HOME=$( cd "${APP_HOME:-./}" && pwd -P ) || exit

# If found unpacked application update move its files to SAYMON Agent directory.
if [ -n "$SAYMON_AGENT_UPDATE_DIRECTORY" ]; then
    SAYMON_AGENT_UPDATE_UNPACK_DIRECTORY="$SAYMON_AGENT_UPDATE_DIRECTORY/saymon-agent-update"
    if [ -d "$SAYMON_AGENT_UPDATE_UNPACK_DIRECTORY" ]; then
        echo "Apply update from $SAYMON_AGENT_UPDATE_UNPACK_DIRECTORY."
        cp -rf "$SAYMON_AGENT_UPDATE_UNPACK_DIRECTORY"/*/* "$APP_HOME"
        rm -rf "$SAYMON_AGENT_UPDATE_UNPACK_DIRECTORY"
    fi
fi

# Add default JVM options here. You can also use JAVA_OPTS and SAYMON_AGENT_OPTS to pass JVM options to this script.
DEFAULT_JVM_OPTS='"-Dfile.encoding=UTF-8" "-Xmx64m" "-XX:+UseSerialGC"'

# Define config file name.
if [ -z "$SAYMON_AGENT_CONFIG_NAME" ]; then
    SAYMON_AGENT_CONFIG_NAME="$APP_HOME/conf/agent.properties"
fi

# Define log directory.
if [ -z "$SAYMON_AGENT_LOGBACK_CONFIGURATION_FILE" ]; then
    SAYMON_AGENT_LOGBACK_CONFIGURATION_FILE="$APP_HOME/conf/logback.xml"
fi

SAYMON_AGENT_OPTS="'-Djava.library.path=$APP_HOME/lib' '-Dlogback.configurationFile=$SAYMON_AGENT_LOGBACK_CONFIGURATION_FILE' '-DSAYMON_AGENT_CONFIG_NAME=$SAYMON_AGENT_CONFIG_NAME'"

# Define update directory.
if [ -n "$SAYMON_AGENT_UPDATE_DIRECTORY" ]; then
    SAYMON_AGENT_OPTS="$SAYMON_AGENT_OPTS '-DSAYMON_AGENT_UPDATE_DIRECTORY=$SAYMON_AGENT_UPDATE_DIRECTORY'"
fi

# Define PID file.
if [ -n "$SAYMON_AGENT_PID_FILE" ]; then
    SAYMON_AGENT_OPTS="$SAYMON_AGENT_OPTS '-DSAYMON_AGENT_PID_FILE=$SAYMON_AGENT_PID_FILE'"
fi

# Use the maximum available, or set MAX_FD != -1 to use that value.
MAX_FD=maximum

warn () {
    echo "$*"
} >&2

die () {
    echo
    echo "$*"
    echo
    exit 1
} >&2

# OS specific support (must be 'true' or 'false').
cygwin=false
msys=false
darwin=false
nonstop=false
case "$( uname )" in                #(
  CYGWIN* )         cygwin=true  ;; #(
  Darwin* )         darwin=true  ;; #(
  MSYS* | MINGW* )  msys=true    ;; #(
  NONSTOP* )        nonstop=true ;;
esac

# Use bundled JRE if available.
if [ -d "$APP_HOME/jre" ]; then
    JAVA_HOME="$APP_HOME/jre"
fi
# Determine the Java command to use to start the JVM.
if [ -n "$JAVA_HOME" ] ; then
    if [ -x "$JAVA_HOME/sh/java" ] ; then
        # IBM's JDK on AIX uses strange locations for the executables
        JAVACMD="$JAVA_HOME/sh/java"
    elif [ -x "$JAVA_HOME/Contents/Home/bin/java" ]; then
        # MacOS
        JAVACMD="$JAVA_HOME/Contents/Home/bin/java"
    else
        JAVACMD="$JAVA_HOME/bin/java"
    fi
    if [ ! -x "$JAVACMD" ] ; then
        die "ERROR: SAYMON Agent is installed without bundled JRE and JAVA_HOME is set to an invalid directory: $JAVA_HOME.

Please install SAYMON Agent distribution with bundled JRE
or set the JAVA_HOME variable in your environment to match the
location of your local Java installation."
    fi
else
    JAVACMD=java
    which java >/dev/null 2>&1 || die "ERROR: SAYMON Agent is installed without bundled JRE, JAVA_HOME is not set and no 'java' command could be found in your PATH.

Please install SAYMON Agent distribution with bundled JRE
or set the JAVA_HOME variable in your environment to match the
location of your local Java installation."
fi

# Increase the maximum file descriptors if we can.
if ! "$cygwin" && ! "$darwin" && ! "$nonstop" ; then
    case $MAX_FD in #(
      max*)
        # In POSIX sh, ulimit -H is undefined. That's why the result is checked to see if it worked.
        # shellcheck disable=SC3045
        MAX_FD=$( ulimit -H -n ) ||
            warn "Could not query maximum file descriptor limit"
    esac
    case $MAX_FD in  #(
      '' | soft) :;; #(
      *)
        # In POSIX sh, ulimit -n is undefined. That's why the result is checked to see if it worked.
        # shellcheck disable=SC3045
        ulimit -n "$MAX_FD" ||
            warn "Could not set maximum file descriptor limit to $MAX_FD"
    esac
fi

# For Cygwin or MSYS, switch paths to Windows format before running java
if "$cygwin" || "$msys" ; then
    APP_HOME=$( cygpath --mixed "$APP_HOME" )
    JAVACMD=$( cygpath --unix "$JAVACMD" )

    # Now convert the arguments - kludge to limit ourselves to /bin/sh
    for arg do
        if
            case $arg in                                #(
              -*)   false ;;                            # don't mess with options #(
              /?*)  t=${arg#/} t=/${t%%/*}              # looks like a POSIX filepath
                    [ -e "$t" ] ;;                      #(
              *)    false ;;
            esac
        then
            arg=$( cygpath --path --ignore --mixed "$arg" )
        fi
        # Roll the args list around exactly as many times as the number of
        # args, so each arg winds up back in the position where it started, but
        # possibly modified.
        #
        # NB: a `for` loop captures its iteration list before it begins, so
        # changing the positional parameters here affects neither the number of
        # iterations, nor the values presented in `arg`.
        shift                   # remove old arg
        set -- "$@" "$arg"      # push replacement arg
    done
fi

# Collect all arguments for the java command, stacking in reverse order:
#   * args from the command line
#   * -jar
#   * DEFAULT_JVM_OPTS, JAVA_OPTS, and SAYMON_AGENT_OPTS environment variables.

set -- -jar "$APP_HOME/lib/saymon-agent.jar" "$@"

# Stop when "xargs" is not available.
if ! command -v xargs >/dev/null 2>&1
then
    die "xargs is not available"
fi

# Use "xargs" to parse quoted args.
#
# With -n1 it outputs one arg per line, with the quotes and backslashes removed.
#
# In Bash we could simply go:
#
#   readarray ARGS < <( xargs -n1 <<<"$var" ) &&
#   set -- "${ARGS[@]}" "$@"
#
# but POSIX shell has neither arrays nor command substitution, so instead we
# post-process each arg (as a line of input to sed) to backslash-escape any
# character that might be a shell metacharacter, then use eval to reverse
# that process (while maintaining the separation between arguments), and wrap
# the whole thing up as a single "set" statement.
#
# This will of course break if any of these variables contains a newline or
# an unmatched quote.
#

eval "set -- $(
        printf '%s\n' "$DEFAULT_JVM_OPTS $JAVA_OPTS $SAYMON_AGENT_OPTS" |
        xargs -n1 |
        sed ' s~[^-[:alnum:]+,./:=@_]~\\&~g; ' |
        tr '\n' ' '
    )" '"$@"'

exec "$JAVACMD" "$@"
